/* ************************************************************************** */
/*                                                                            */
/*                                                        :::      ::::::::   */
/*   main.c                                             :+:      :+:    :+:   */
/*                                                    +:+ +:+         +:+     */
/*   By: cchauvet <cchauvet@student.42angoulem      +#+  +:+       +#+        */
/*                                                +#+#+#+#+#+   +#+           */
/*   Created: 2023/02/16 15:16:14 by cchauvet          #+#    #+#             */
/*   Updated: 2023/02/20 15:24:42 by starnakin        ###   ########.fr       */
/*                                                                            */
/* ************************************************************************** */

#include "libftx/libftx.h"
#include "minishell.h"

static char	*ft_get_user_input(t_list **env)
{
	char	*line;
	char	*prompt;

	prompt = ft_strmerger(2, get_value_by_key("PWD", env), "$ ");
	if (prompt == NULL)
	{
		ft_eprintf("minishell: malloc failed\n");
		return (NULL);
	}
	line = readline(prompt);
	add_history(line);
	free(prompt);
	return (line);
}

static int	ft_minishell(t_list **env, char *line)
{
	t_list	**cmds;
	int		code;
	char	*line_clean;
	int		infile;
	int		outfile;

	line_clean = ft_normalizer(line);
	if (line_clean == NULL)
		return (1);
	if (ft_syntatic_verif(line_clean))
	{
		free(line_clean);
		return (1);
	}
	outfile = ft_outfile(line_clean);
	if (outfile == -2)
	{
		free(line_clean);
		return (1);
	}
	infile = ft_infile(line_clean);
	if (infile == -2)
	{
		close(outfile);
		free(line_clean);
		return (1);
	}
	cmds = ft_parse_cmds(line_clean, env, infile, outfile);
	if (cmds == NULL)
	{
		close(outfile);
		close(infile);
		ft_lstclear(cmds, ft_cmddel);
		free(cmds);
		free(line_clean);
		return (1);
	}
	code = ft_cmds_executor(cmds, env);
	ft_lstclear(cmds, ft_cmddel);
	free(cmds);
	free(line_clean);
	return (code);
}

#if DEBUG

int	main(int ac, char **av, char **env)
{
	t_data	data;

	if (ac == 1)
		return (1);
	data.env = init_env(env);
	if (data.env == NULL)
		return (1);
	if (ft_minishell(data.env, av[1]) == 1)
	{
		ft_lstclear(data.env, env_del);
		free(data.env);
		return (1);
	}
	ft_lstclear(data.env, env_del);
	free(data.env);
	return (0);
}

#else

int	main(int ac, char **av, char **env)
{
	t_data	data;
	char	*line;

	data.env = init_env(env);
	if (data.env == NULL)
		return (1);
	create_value_by_key("", "$", data.env);
	line = ft_get_user_input(data.env);
	if (line == NULL)
	{
		ft_lstclear(data.env, env_del);
		free(data.env);
		return (1);
	}
	while (line != NULL)
	{
		if (ft_minishell(data.env, line) == 2)
			break ;
		free(line);
		line = ft_get_user_input(data.env);
		if (line == NULL)
		{
			ft_lstclear(data.env, env_del);
			free(data.env);
			return (1);
		}
	}
	ft_lstclear(data.env, env_del);
	free(data.env);
}

#endif
